"use strict";
var application = require("application");
var common = require("./carousel-view-common");
var VIEWS_STATES = "_viewStates";
var CarouselView = (function (_super) {
    __extends(CarouselView, _super);
    function CarouselView() {
        return _super.apply(this, arguments) || this;
    }
    Object.defineProperty(CarouselView.prototype, "android", {
        get: function () {
            return this._android;
        },
        enumerable: true,
        configurable: true
    });
    CarouselView.prototype._createUI = function () {
        this._android = new android.widget.RelativeLayout(application.android.currentContext);
    };
    CarouselView.prototype.onLoaded = function () {
        if (this.orientation == 0)
            this._viewPager = new android.support.v4.view.ViewPager(this._context);
        else {
            ensureVerticalViewPagerClass();
            this._viewPager = new VerticalViewPagerClass(application.android.currentContext);
        }
        var res = android.content.res.Resources;
        var margin = this.interPageSpacing * res.getSystem().getDisplayMetrics().density;
        this._viewPager.setPageMargin(margin);
        var that = new WeakRef(this);
        ensurePagerAdapterClass();
        this._viewPager.setAdapter(new PagerAdapterClass(this));
        ensurePageChangedListenerClass();
        this._viewPager.setOnPageChangeListener(new PageChangedListenerClass(this));
        this._viewPager.setCurrentItem(this.position, false);
        var layoutParams = new android.widget.RelativeLayout.LayoutParams(android.widget.RelativeLayout.LayoutParams.MATCH_PARENT, android.widget.RelativeLayout.LayoutParams.MATCH_PARENT);
        this._android.addView(this._viewPager, layoutParams);
        var eventData = {
            eventName: "positionSelected",
            object: this
        };
        this.notify(eventData);
    };
    CarouselView.prototype.insertPage = function (position, bindingContext) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this._viewPager != null))
                            return [3 /*break*/, 2];
                        if (position > this.itemsSource.length)
                            throw new Error("Index out of bounds (position > itemsSource length).");
                        if (position < 0)
                            throw new Error("Index out of bounds (position < 0).");
                        if (position == this.itemsSource.length)
                            this.itemsSource.push(bindingContext);
                        else
                            this.itemsSource.splice(position, 0, bindingContext);
                        if (position > 1)
                            this._viewPager.getAdapter().notifyDataSetChanged();
                        else
                            this._viewPager.setAdapter(new PagerAdapterClass(this));
                        return [4 /*yield*/, this.delay(100)];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    CarouselView.prototype.removePage = function (position) {
        return __awaiter(this, void 0, void 0, function () {
            var newPos;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this._viewPager != null))
                            return [3 /*break*/, 6];
                        if (position > this.itemsSource.length - 1)
                            throw new Error("Index out of bounds (position > itemsSource length - 1).");
                        if (position < 0)
                            throw new Error("Index out of bounds (position < 0).");
                        if (!(position == this.position))
                            return [3 /*break*/, 5];
                        newPos = position - 1;
                        if (newPos == -1)
                            newPos = 0;
                        if (!(position == 0))
                            return [3 /*break*/, 2];
                        this._viewPager.setCurrentItem(1, true);
                        return [4 /*yield*/, this.delay(100)];
                    case 1:
                        _a.sent();
                        this.itemsSource.splice(position, 1);
                        //this._viewPager.setAdapter(new PagerAdapterClass(this));
                        this._viewPager.getAdapter().notifyDataSetChanged();
                        this._viewPager.setCurrentItem(0, false);
                        this.position = 0;
                        return [3 /*break*/, 4];
                    case 2:
                        this._viewPager.setCurrentItem(newPos, true);
                        return [4 /*yield*/, this.delay(100)];
                    case 3:
                        _a.sent();
                        this.itemsSource.splice(position, 1);
                        if (position == 1)
                            this._viewPager.setAdapter(new PagerAdapterClass(this));
                        else
                            this._viewPager.getAdapter().notifyDataSetChanged();
                        this.position = newPos;
                        _a.label = 4;
                    case 4: return [3 /*break*/, 6];
                    case 5:
                        this.itemsSource.splice(position, 1);
                        if (position == 1)
                            this._viewPager.setAdapter(new PagerAdapterClass(this));
                        else
                            this._viewPager.getAdapter().notifyDataSetChanged();
                        _a.label = 6;
                    case 6: return [2 /*return*/];
                }
            });
        });
    };
    CarouselView.prototype.setCurrentPage = function (position) {
        if (this._viewPager != null) {
            if (position > this.itemsSource.length - 1)
                throw new Error("Index out of bounds (position > itemsSource length - 1).");
            if (position < 0)
                throw new Error("Index out of bounds (position < 0).");
            this.position = position;
            this._viewPager.setCurrentItem(position, true);
        }
    };
    CarouselView.prototype.itemsSourceChanged = function () {
        if (this._viewPager != null) {
            if (this.position > this.itemsSource.length - 1)
                this.position = this.itemsSource.length - 1;
            ensurePagerAdapterClass();
            this._viewPager.setAdapter(new PagerAdapterClass(this));
            this._viewPager.setCurrentItem(this.position, false);
            var eventData = {
                eventName: "positionSelected",
                object: this
            };
            this.notify(eventData);
        }
    };
    CarouselView.prototype.delay = function (ms) {
        return new Promise(function (resolve) { return setTimeout(resolve, ms); });
    };
    return CarouselView;
}(common.CarouselView));
exports.CarouselView = CarouselView;
var PagerAdapterClass;
function ensurePagerAdapterClass() {
    if (PagerAdapterClass) {
        return;
    }
    var PagerAdapterInner = (function (_super) {
        __extends(PagerAdapterInner, _super);
        function PagerAdapterInner(_owner) {
            var _this = _super.call(this) || this;
            _this._owner = _owner;
            return global.__native(_this);
        }
        PagerAdapterInner.prototype.getCount = function () {
            // FIX: populate the carousel with data after being loaded in UI
            if (this._owner.itemsSource == null)
                return 0;
            return this._owner.itemsSource.length;
        };
        PagerAdapterInner.prototype.isViewFromObject = function (view, objectValue) {
            return view === objectValue;
        };
        PagerAdapterInner.prototype.instantiateItem = function (container, position) {
            // FIX: populate the carousel with data after being loaded in UI
            var item;
            if (this._owner.itemsSource != null)
                item = this._owner.itemsSource.getItem(position);
            var view = this._owner.templateSelector.OnSelectTemplate(position, item);
            var obj = view;
            //obj._onAttached(application.android.currentContext);
            this._owner._addView(obj);
            obj.android.tag = position;
            if (this[VIEWS_STATES]) {
                obj.android.restoreHierarchyState(this[VIEWS_STATES]);
            }
            container.addView(obj.android);
            obj.onLoaded();
            return obj.android;
        };
        PagerAdapterInner.prototype.destroyItem = function (container, position, objectValue) {
            var pager = container;
            pager.removeView(objectValue);
        };
        PagerAdapterInner.prototype.getItemPosition = function (objectValue) {
            var tag = Number(objectValue.getTag());
            var position = this._owner.position;
            if (tag == position)
                return tag;
            return android.support.v4.view.PagerAdapter.POSITION_NONE;
        };
        PagerAdapterInner.prototype.saveState = function () {
            if (!this[VIEWS_STATES]) {
                this[VIEWS_STATES] = new android.util.SparseArray();
            }
            var mViewStates = this[VIEWS_STATES];
            var mViewPager = this._owner.android;
            var count = mViewPager.getChildCount();
            for (var i = 0; i < count; i++) {
                var c = mViewPager.getChildAt(i);
                if (c.isSaveFromParentEnabled()) {
                    c.saveHierarchyState(mViewStates);
                }
            }
            var bundle = new android.os.Bundle();
            bundle.putSparseParcelableArray(VIEWS_STATES, mViewStates);
            return bundle;
        };
        PagerAdapterInner.prototype.restoreState = function (state, loader) {
            var bundle = state;
            bundle.setClassLoader(loader);
            this[VIEWS_STATES] = bundle.getSparseParcelableArray(VIEWS_STATES);
        };
        return PagerAdapterInner;
    }(android.support.v4.view.PagerAdapter));
    PagerAdapterClass = PagerAdapterInner;
}
var PageChangedListenerClass;
function ensurePageChangedListenerClass() {
    if (PageChangedListenerClass) {
        return;
    }
    var PageChangedListenerInner = (function (_super) {
        __extends(PageChangedListenerInner, _super);
        function PageChangedListenerInner(owner) {
            var _this = _super.call(this) || this;
            _this._owner = owner;
            return global.__native(_this);
        }
        PageChangedListenerInner.prototype.onPageSelected = function (position) {
            this._owner.position = position;
        };
        PageChangedListenerInner.prototype.onPageScrollStateChanged = function (state) {
            if (state == 0) {
                var eventData = {
                    eventName: "positionSelected",
                    object: this._owner
                };
                this._owner.notify(eventData);
            }
        };
        return PageChangedListenerInner;
    }(android.support.v4.view.ViewPager.SimpleOnPageChangeListener));
    PageChangedListenerClass = PageChangedListenerInner;
}
var VerticalViewPagerClass;
function ensureVerticalViewPagerClass() {
    if (VerticalViewPagerClass) {
        return;
    }
    var VerticalViewPagerInner = (function (_super) {
        __extends(VerticalViewPagerInner, _super);
        function VerticalViewPagerInner(context, attrs) {
            var _this = _super.call(this, context, attrs) || this;
            global.__native(_this).setPageTransformer(false, new android.support.v4.view.ViewPager.PageTransformer({
                transformPage: function (page, position) {
                    page.setTranslationX(page.getWidth() * -position);
                    var yPosition = position * page.getHeight();
                    page.setTranslationY(yPosition);
                }
            }));
            // get rid of the overscroll drawing that happens on the left and right
            global.__native(_this).setOverScrollMode(android.view.View.OVER_SCROLL_NEVER);
            return global.__native(_this);
        }
        VerticalViewPagerInner.prototype.swapTouchEvent = function (ev) {
            var width = this.getWidth();
            var height = this.getHeight();
            var swappedX = (ev.getY() / height) * width;
            var swappedY = (ev.getX() / width) * height;
            ev.setLocation(swappedX, swappedY);
            return ev;
        };
        VerticalViewPagerInner.prototype.onInterceptTouchEvent = function (ev) {
            var intercept = _super.prototype.onInterceptTouchEvent.call(this, this.swapTouchEvent(ev));
            //If not intercept, touch event should not be swapped.
            this.swapTouchEvent(ev);
            return intercept;
        };
        VerticalViewPagerInner.prototype.onTouchEvent = function (ev) {
            return _super.prototype.onTouchEvent.call(this, this.swapTouchEvent(ev));
        };
        return VerticalViewPagerInner;
    }(android.support.v4.view.ViewPager));
    VerticalViewPagerClass = VerticalViewPagerInner;
}
//# sourceMappingURL=carousel-view.android.js.map